<?php
/**
 * Plugin Name: Icecast Stream Info
 * Plugin URI: https://prahec.com/
 * Description: Custom WordPress plugin that allows you to use shortcodes for real-time icecast info
 * Version: 1.1
 * Author: Jacky
 * Author URI: http://prahec.com/
 */

// Wordpress stuff
add_action('init', 'load_jquery_ifnot_js');
add_shortcode('iceinfo', 'icecast_streaminfo_shortcode');
add_action('wp_head', 'icecast_straminfo_head');

add_action('bp_before_member_header_meta', 'icecast_streaminfo_userpage');
add_action('bp_directory_members_item', 'icecast_streaminfo_buddypress');

function load_jquery_ifnot_js() {
	wp_enqueue_script( 'jquery' );
}

// If BuddyPress is installed, this function will display iceinfo on users profile
function icecast_streaminfo_userpage() {
	
	global $bp;
	
	$cUserID = $bp->displayed_user->id;
	
	$stationName = xprofile_get_field_data('Station', $cUserID);
	$streamURL = xprofile_get_field_data('Stream URL', $cUserID);
	
	$streamURL = preg_match('/(http:\/\/)?([a-zA-Z0-9\:\.\-]+)\/(.*)/i', $streamURL, $iceinfoURL);
	
	if (!empty($iceinfoURL[2]) AND !empty($iceinfoURL[3])) {
		
		echo '<b>' . $stationName . '</b><br />
		<div style="margin: 0; padding: 0;" id="' . uniqid() . '" class="iceInfo" data-server="http://' . $iceinfoURL[2] . '/status.xsl?mount=/' . $iceinfoURL[3] . '" data-info="genre,nowplaying">Loading info...</div>';
		
	}
	
}

function icecast_streaminfo_buddypress() {
	
	global $uniqueAudioID;
	
	$iceStation = bp_get_member_profile_data('field=Station');
	$iceURL = bp_get_member_profile_data('field=Stream URL');
	
	preg_match('/(http:\/\/)?([a-zA-Z0-9\:\.\-]+)\/(.*)/i', $iceURL, $iceinfoURL);
	
	if (!empty($iceinfoURL[2]) AND !empty($iceinfoURL[3])) {
		
		// If uniqueID empty, generate new one, else just increse it
		if (empty($uniqueAudioID)) { $uniqueAudioID = 1; } else { $uniqueAudioID++; }
		
		echo '<div class="userRadio"><b>' . $iceStation . '</b><br />' .
			'<div style="margin: 0; padding: 0;" id="iceinfo-' . $uniqueAudioID . '" class="iceInfo" data-server="http://' . $iceinfoURL[2]
			. '/status.xsl?mount=/' . $iceinfoURL[3] . '" data-info="genre,nowplaying">Loading info...</div></div>';
		
	}
	
}

function icecast_straminfo_head() {
?>

<script type="text/javascript">
	jQuery(function($) {
		
		$(document).ready(function() {
			
			var $allIces = {};
			var iceinfoNames = {
				'streamtitle': 'Stream Title',
				'streamdesc' : 'Description',
				'streamurl'	 : 'URL',
				'codec'		 : 'Codec',
				'bitrate'	 : 'Bitrate',
				'listeners'	 : 'Current Listeners',
				'peaks'		 : 'Peak Listeners',
				'genre'		 : 'Genre',
				'nowplaying' : 'Playing'
			}
			
			// First get all servers to parse
			$(document).find('.iceInfo').each(function(e) {
				
				var uniqID = $(this).attr('id');
				$allIces[uniqID] = $(this).attr('data-server');
				
			});
			
			function iceinfo_ajaxcall() {
				
				// Do ajax POST for all shortcodes
				$.ajax('/?action=query-icecast&return=data', {
					
					data: { iceinfo: $allIces },
					dataType: 'json',
					type: 'POST',
					async: false,
					cache: false,
					
				}).done(function(data) {
					
					$.each(data, function(statsID, iceinfo) {
						
						// Break show options into array
						var showOpts = $('#' + statsID).attr('data-info');
						var allOpts = showOpts.split(',');
						
						// Clear DIV before appending new stuff
						$('#' + statsID).empty();
						
						// Loop through show option and display only things specified					
						$.each(allOpts, function(index, value) {
							if (iceinfo[value] != null) {
								$('#' + statsID).append('<b>' + iceinfoNames[value] + '</b>: ' + iceinfo[value] + '<br />');
							}
						});
						
					});
					
				});
				
			}
			
			if ($('.iceInfo').length) {
				// Call Ajax & Set interval to 20sec (periodic check)
				iceinfo_ajaxcall();
				setInterval(iceinfo_ajaxcall, 20000);
			}
			
		});
		
	});
</script>
<?php	
}

function icecast_streaminfo_shortcode($data) {
	
	global $uniqueAudioID;
	
	extract(shortcode_atts(array(
		'url' => null,
		'mount' => null,
		'show'	=> 'nowplaying, genre'
	), $data));
	
	// If uniqueID empty, generate new one, else just incresse it
	if (empty($uniqueAudioID)) { $uniqueAudioID = 1; } else { $uniqueAudioID++; }
	
	$html = '<div id="iceinfo-' . $uniqueAudioID . '" class="iceInfo" data-server="' . $url . '/status.xsl?mount=/' . $mount . '" data-info="' . $show . '">Loading info...</div>';
	
	if (!empty($url) AND !empty($mount)) {
		return $html;
	} else {
		return false;
	}
}

// Function which switches between cURL & PHP file_get_contents to get data from remote server
function iceinfo_connect($openURL, $auth = '') {
	
	if(function_exists('curl_version')) { // if CURL is enabled
		
		$cUrl = curl_init();
		curl_setopt($cUrl, CURLOPT_URL, $openURL);
		curl_setopt($cUrl, CURLOPT_RETURNTRANSFER, 1 );
		curl_setopt($cUrl, CURLOPT_USERAGENT, "Mozilla");
		
		if (!empty($auth)) {
			curl_setopt($cUrl, CURLOPT_USERPWD, $auth);
		}
		
		$data = curl_exec($cUrl);
		curl_close($cUrl);
		
	} else { // Slower but works on CURL disabled servers
		
		if (!empty($auth)) {
			$a = stream_context_create(array('http' => array('header'  => "Authorization: Basic " . base64_encode($auth))));
		}    
		
		$data = file_get_contents($openURL, $false, $a);
		
	}
	
	return $data;
	
}

// Function to parse Icecast HTML response (status.xsl) 
function iceinfo_parse($data) {
	
	$radioInfo = array();
	$tempArray = array();
	
	// Match all td's and get data from them
	if(preg_match_all("/<td\s[^>]*class=\"streamdata\">(.*)<\/td>/siU",$data,$matches)) {
		foreach($matches[0] as $match) {
			$to_push = str_replace(array('<td class="streamdata">','</td>'),'',$match);
			$to_push = trim($to_push);
			array_push($tempArray,$to_push);
		}
	}
	
	return array(
		'streamtitle' 		=>	$tempArray['0'],
		'streamdesc'		=>	$tempArray['1'],
		'codec'				=>	$tempArray['2'],
		'streamurl'		    =>	$tempArray['8'],
		'bitrate'			=>  $tempArray['4'],
		'listeners'			=>	$tempArray['5'],
		'peaks'				=>  $tempArray['6'],
		'genre'				=>	$tempArray['7'],
		'nowplaying'		=>	$tempArray['9']
	);
	
}


// Here we place handler for the queries & responses
if (isset($_POST['iceinfo']) AND $_GET['action'] == 'query-icecast') {
	
	// Close SESSION and prevent caching
	session_write_close();
	
	if (is_array($_POST['iceinfo'])) { // Check if post is array
		
		$newData = array();
		
		foreach ($_POST['iceinfo'] as $key => $url) { // Query all and create new array to return
			
			// Query server
			$newData[$key] = iceinfo_parse(iceinfo_connect($url));
			
		}
		
		
		echo json_encode($newData);
		
	}
	
	exit;
	
}

?>